@@ -22,7 +22,6 @@ from amqp import RecoverableConnectionError
 from .entity import Exchange, Queue
 from .five import range
 from .log import get_logger
-from .messaging import Consumer as _Consumer
 from .serialization import registry as serializers
 from .utils import uuid
 
@@ -91,33 +90,40 @@ def declaration_cached(entity, channel):
 
 
 def maybe_declare(entity, channel=None, retry=False, **retry_policy):
-    if not entity.is_bound:
-        assert channel
-        entity = entity.bind(channel)
-    if retry:
-        return _imaybe_declare(entity, **retry_policy)
-    return _maybe_declare(entity)
+    is_bound = entity.is_bound
 
+    if channel is None:
+        assert is_bound
+        channel = entity.channel
 
-def _maybe_declare(entity):
-    channel = entity.channel
-    if not channel.connection:
-        raise RecoverableConnectionError('channel disconnected')
-    if entity.can_cache_declaration:
+    declared = ident = None
+    if channel.connection and entity.can_cache_declaration:
         declared = channel.connection.client.declared_entities
         ident = hash(entity)
-        if ident not in declared:
-            entity.declare()
-            declared.add(ident)
-            return True
-        return False
+        if ident in declared:
+            return False
+
+    entity = entity if is_bound else entity.bind(channel)
+    if retry:
+        return _imaybe_declare(entity, declared, ident,
+                               channel, **retry_policy)
+    return _maybe_declare(entity, declared, ident, channel)
+
+
+def _maybe_declare(entity, declared, ident, channel):
+    channel = channel or entity.channel
+    if not channel.connection:
+        raise RecoverableConnectionError('channel disconnected')
     entity.declare()
+    if declared is not None and ident:
+        declared.add(ident)
     return True
 
 
-def _imaybe_declare(entity, **retry_policy):
+def _imaybe_declare(entity, declared, ident, channel, **retry_policy):
     return entity.channel.connection.client.ensure(
-        entity, _maybe_declare, **retry_policy)(entity)
+        entity, _maybe_declare, **retry_policy)(
+            entity, declared, ident, channel)
 
 
 def drain_consumer(consumer, limit=1, timeout=None, callbacks=None):
@@ -138,8 +144,8 @@ def drain_consumer(consumer, limit=1, timeout=None, callbacks=None):
 
 
 def itermessages(conn, channel, queue, limit=1, timeout=None,
-                 Consumer=_Consumer, callbacks=None, **kwargs):
-    return drain_consumer(Consumer(channel, queues=[queue], **kwargs),
+                 callbacks=None, **kwargs):
+    return drain_consumer(conn.Consumer(channel, queues=[queue], **kwargs),
                           limit=limit, timeout=timeout, callbacks=callbacks)
 
 
